﻿--[[
Hunter AEP version 0.0.7
by Kevin Clement <kevin.clement@gmail.com> (Arcuss)

Created: Febuary 9, 2007
Last Updated: April 5, 2007

About:	Agility Equivalence Points

This mod is used to calculate Agility Equivalence Points for an item in
the tool tip.  It uses formulas calculated by Arcazua on TKASomething forum.

TKASomething Post
http://forums.tkasomething.com/viewtopic.php?t=3824

Use /haep to set attribute weights.

Formula from TKASomething post:
	Agi - 10 points 
	Int - 5 points 
	AP - 6 points 
	CR - 8 points 
	HR - 13 points 
	MP5 - 20 points 

Thanks:
	Arcazua on TKASomething for forumulas
	lythrdskynrd,Nazkur on TKASomething for testing and feedback
	Jubei's Mod for original inspiration
	Jerry from ACE Mods for ItemBonusLib and support

History:
 v0.0.7 Updated German translation
 v0.0.6 Added GUI based changing of options via /haep
		Added ability to reset to default weights
		Added ability to add weight of stamina
		Added ability to add weight of Resilience
		Fixed some typos
 v0.0.5 Updated to latest ItemBonusLib that implemented my fixes officially
        Updated Some Ace Libraries and cleaned up TOC
        Added localization support with US and German already supported.  (Thanks to Nazkur)
 v0.0.4 Fixed some german translation errors in ItemBonusLib
 v0.0.3 Complete rewrite into ACE mod, this should:
            Fix bugs with socketed items
            Handle hooking of the tooltip better, less calls
            Allow for enabling/disabling via command line, see /haep
            Allow for setting of values via command line, see /haep
            More robust for future upgrades/features
        Added a Total column to the right
 v0.0.2 Update values to reflect discussion.
 v0.0.1 Initial release with code stripped out.
]]
HunterAEP = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceDebug-2.0")
HunterAEP.version = "0.0.7"
HunterAEP.L = AceLibrary("AceLocale-2.2"):new("HunterAEP")

local Bonus = AceLibrary("ItemBonusLib-1.0")
local Deformat = AceLibrary("Deformat-2.0")
local TipHooker = AceLibrary("TipHooker-1.0")
local L = HunterAEP.L

-- Enable or disable debugging
HunterAEP:SetDebugging(false)

-- Defaults for all the types, I intentially left out 
--   RANGEDCRIT
--   TOHIT
-- I think these are the old preTBC 
local profileDefaults = {
	AGI					= 10,
	INT					= 5,
	ATTACKPOWER			= 6,
	RANGEDATTACKPOWER	= 6,
	CR_HIT				= 13,
	CR_CRIT				= 8,
	MANAREG				= 20,
	STA					= 0,
	CR_RESILIENCE		= 0,

	TotalDisplayed		= true,
}

HunterAEP:RegisterDB("HunterAEPDB")
HunterAEP:RegisterDefaults('profile', profileDefaults)

---------------------------------------------------------------------------------------
-- Toggles the config window on and off
---------------------------------------------------------------------------------------
function HunterAEPOptions_Toggle()
	if(HunterAEPOptionsFrame:IsVisible()) then
		HunterAEPOptionsFrame:Hide();
	else
		HunterAEPOptionsFrame:Show();
	end
end

function HunterAEP:OnInitialize()
    -- Called when the addon is loaded

	-- Register simple slash command
	SlashCmdList["HUNTERAEP"] = HunterAEPOptions_Toggle;
	SLASH_HUNTERAEP1 = "/haep";
	SLASH_HUNTERAEP2 = "/hunteraep";
end

function HunterAEP:OnEnable()
    -- Called when the addon is enabled
    TipHooker:Hook(self.ProcessTooltip, "item")
end

function HunterAEP:OnDisable()
    -- Called when the addon is disabled
    TipHooker:Unhook(self.ProcessTooltip, "item")
end

---------------------------------------------------------------------------------------
-- Pulls the Item string out of the link string, this is pulled from wowwiki
-- if it had problems, just return the orginal link
-- 
-- link: larget link string with formatting and such
---------------------------------------------------------------------------------------
local function ExtractItemString(link)
	local found, _, itemString = string.find(link, "^|%x+|H(.+)|h%[.+%]")
	return itemString or link
end

---------------------------------------------------------------------------------------
-- Takes a link and will pull out the item string, and strip the enchant from it
-- It will return the new itemString without enchant
-- 
-- link: larget link string with formatting and such
---------------------------------------------------------------------------------------
local function ClearEnchantsFromLink(link)
	local fullItemString = ExtractItemString(link)
	local matchString = "^(item:%d+:)%d+:"
	return string.gsub(fullItemString, matchString, "%10:")
end

---------------------------------------------------------------------------------------
-- Takes a link and will pull out the item string, and strip the sockets from it
-- It will return the new itemString without sockets
-- 
-- link: larget link string with formatting and such
---------------------------------------------------------------------------------------
local function ClearSocketsFromLink(link)
	local fullItemString = ExtractItemString(link)
	local matchString = "^(item:%d+:%d+:)%d+:%d+:%d+:%d+:"
	return string.gsub(fullItemString, matchString, "%10:0:0:0:")
end

---------------------------------------------------------------------------------------
-- Used in debugging to print the stats of a moused over object
--
-- infoObj: already parsed info object
-- objType: what type of object is this
---------------------------------------------------------------------------------------
function HunterAEP:PrintStats(infoObj, objType)
	if not self:IsDebugging() then return end
	self:Debug(objType.." ---------------------")
	for bonus, value in pairs(infoObj.bonuses) do
		self:Debug(bonus.." : "..value)
	end
end

---------------------------------------------------------------------------------------
-- Looks up the AEP value of the given bonus, if it isnt specified then return 0
---------------------------------------------------------------------------------------
function HunterAEP:LookupAEP(bonus, value)
	if(self.db.profile[bonus]) then
		return self.db.profile[bonus]*value
	else
		return 0
	end
end

---------------------------------------------------------------------------------------
-- Used to calculate and AEP given an info object that came out of ItemBonusLib
--
-- infoObj: already parsed info object
---------------------------------------------------------------------------------------
function HunterAEP:CalculateAEP(infoObj)
	local aepTotal = 0

	-- go through each bonus, if we find it in our 
	-- profile with a value then multiply it
	for bonus, value in pairs(infoObj.bonuses) do
		aepTotal = aepTotal + HunterAEP:LookupAEP(bonus,value)
	end

	return aepTotal
end

---------------------------------------------------------------------------------------
-- Formulates the display string to be used in the tooltip, can potentially be 2 parts
---------------------------------------------------------------------------------------
function HunterAEP:FormulateDisplayStrings(aepBase, aepOriginal, aepEnchant, aepSocket)
	local leftString = ""

	local leftString = string.format("AEP: %s", aepBase)
	if aepEnchant>0 then
		leftString = string.format("%s %s(+%s)|r", leftString, GREEN_FONT_COLOR_CODE, aepEnchant)
	end
	if aepSocket>0 then
		leftString = string.format("%s %s(+%s)|r", leftString, HIGHLIGHT_FONT_COLOR_CODE, aepSocket)
	end

	return leftString
end

---------------------------------------------------------------------------------------
-- Loops through the active tooltip and will pull out the set count 
-- e.g. Striker's Garb (3/5) => 3
---------------------------------------------------------------------------------------
function HunterAEP:GetSetCount(tooltip)
	for i=2, tooltip:NumLines() do
		local line = getglobal(tooltip:GetName() .. "TextLeft" .. i):GetText()
		set_name, set_count, set_total = Deformat(line, ITEM_SET_NAME)
		if set_name then
			return set_count
		end
	end
	return 0
end

---------------------------------------------------------------------------------------
-- Runs whenever a tooltip is displayed.  This is controlled by TipHooker and calls into
-- this method.
--
-- tooltip: Tooltip object (table)
-- name: Name of the item (string)
-- link: ItemLink (string)
---------------------------------------------------------------------------------------
function HunterAEP.ProcessTooltip(tooltip, name, link)
	local origInfo, enchantInfo, socketInfo
	local aepOriginal, aepEnchant, aepSocket, aepBase, aepGrandTotal = 0
	local linesToAdd = { }

	-- return if this is a tooltip without a name, happens on empty inventory
	if not name then return end

	-- grab the bonus info: standard scan
	-- if there were problems return nothing
	origInfo = Bonus:ScanItemLink(link)
	if not origInfo then return end
	aepOriginal = HunterAEP:CalculateAEP(origInfo)
	aepGrandTotal = aepOriginal

	-- if there wasnt a total and there isnt
	-- the potential for set bonus, then return
	if aepOriginal <= 0 and not origInfo.set then return end

	-- prints out the stats on the item if debugging is enabled
	HunterAEP:PrintStats(origInfo, "Orig Info")

	-- get an info object that is the item without enchants
	enchantInfo=Bonus:ScanItemLink(ClearEnchantsFromLink(link))
	if not enchantInfo then return end
	aepEnchant = aepOriginal - HunterAEP:CalculateAEP(enchantInfo)

	-- get an info object that is the item without sockets
	socketInfo=Bonus:ScanItemLink(ClearSocketsFromLink(link))
	if not socketInfo then return end
	aepSocket = aepOriginal - HunterAEP:CalculateAEP(socketInfo)
	
	-- the base is the original minus the sockets and enchants
	aepBase = aepOriginal - aepEnchant - aepSocket

	-- formulate display strings
	local leftString=HunterAEP:FormulateDisplayStrings(aepBase, aepOriginal, aepEnchant, aepSocket)

	-- handle sets
	if origInfo.set then
		local setCount = HunterAEP:GetSetCount(tooltip)

		local setBonuses = Bonus:GetSetBonuses(origInfo.set)

		-- find out the set stats
		-- display as grey if we havent met the stats yet
		-- otherwise display as normal on new line but add to the total
		for setNum, bonus in pairs(setBonuses) do
			for bonusName, value in pairs(bonus) do
				if setNum <= setCount then
					local total = HunterAEP:LookupAEP(bonusName,value)
					if total > 0 then
						-- set string is "Set: %s"
						local setText = string.format(ITEM_SET_BONUS, "+"..total.."|r")
						table.insert(linesToAdd, setText)
						aepGrandTotal = aepGrandTotal + total
					end
				else
					local total=HunterAEP:LookupAEP(bonusName,value)
					if total > 0 then
						-- set string is "(%d) Set: %s"
						local setText = string.format(ITEM_SET_BONUS_GRAY, setNum, "+"..HunterAEP:LookupAEP(bonusName,value))
						table.insert(linesToAdd, string.format("%s%s|r", GRAY_FONT_COLOR_CODE, setText))
					end
				end
			end
		end
	end

	local rightString = ""
	if aepBase ~= aepGrandTotal  then
		rightString = "Total: "..aepGrandTotal
	end 
	if not HunterAEP.db.profile['TotalDisplayed'] then
		rightString = ""
	end


	if aepGrandTotal == 0 and getn(linesToAdd) == 0 then return end

	-- add all the lines we accumulated
	tooltip:AddLine(" ")
	tooltip:AddDoubleLine(leftString, rightString)
	for _,line in pairs(linesToAdd) do
		tooltip:AddLine(line)
	end

	tooltip:Show()
end